/*
 * Test pipe operations.
 *
 * Copyright (C) 2020, 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/sys/err.h>

#include <stdio.h>

#include "memory.h"
#include "pages_conserving.h"
#include "pipe_opener_resource.h"
#include "resource_server.h"



const unsigned int MEMORY_PAGES = 20;
const unsigned int PIPE_PAGES = 2;

int main(void)
{
  /* Some memory plus infrastructure. */

  Memory mem(MEMORY_PAGES);
  PagesConserving pages(&mem, PIPE_PAGES);
  PipeOpenerResource opener(&pages);

  /* Register a server associating it with the given object. */

  ResourceServer server(&opener);
  long err = server.bind("server");

  if (err)
  {
    printf("Could not bind server: %s\n", l4sys_errtostr(err));
    return 1;
  }

  printf("Starting server...\n");
  server.start();
  return 0;
}
