#include "pages_conserving.h"

PagesConserving::PagesConserving(Memory *memory, unsigned int increment)
: Pages(memory), _increment(increment)
{
}

PagesConserving::PagesConserving()
: Pages(), _increment(1)
{
}



/* Refuse to obtain a new flexpage directly. All flexpages must be obtained from
   the page queue. */

Flexpage *PagesConserving::flexpage()
{
    return NULL;
}

/* Pretend to queue an entry associating the given 'owner' and 'flexpage'.
   Since this collection "conserves" flexpages, they will not be queued unless
   completely unused. */

void PagesConserving::queue(PageOwner *owner, Flexpage *flexpage)
{
    (void) owner; (void) flexpage;
}

/* Pretend to reserve a flexpage already being used by 'owner' and 'flexpage'.
   Since flexpages are not queued when in use, there is nothing to reserve. */

bool PagesConserving::reserve(PageOwner *owner, Flexpage *flexpage)
{
    (void) owner; (void) flexpage;
    return true;
}



/* Decrease the provision of flexpages to pipes. */

bool PagesConserving::decrease()
{
    // NOTE: Need to remove flexpages from the queue.
    // NOTE: This might need to happen incrementally in case some pages are not
    // NOTE: back in the queue for whatever reason.

    return true;
}

/* Increase the provision of flexpages to pipes, returning whether allocation
   succeeded. */

bool PagesConserving::increase()
{
    unsigned int regions = _increment;
    std::list<Region *> allocated;
    std::list<Region *>::iterator it;

    /* Allocate regions for the defined increment value. */

    while (regions)
    {
        Region *r = _memory->region();

        /* Free all allocated regions if not all regions can be allocated. */

        if (r == NULL)
        {
            for (it = allocated.begin(); it != allocated.end(); it++)
                _memory->release(r);

            return false;
        }

        allocated.push_back(r);
        regions--;
    }

    /* Queue flexpages for the regions. */

    for (it = allocated.begin(); it != allocated.end(); it++)
        _queue.push(NULL, new Flexpage(*it));

    return true;
}

/* Return the allocation increment. */

offset_t PagesConserving::allocation()
{
    return _memory->region_size() * _increment;
}

// vim: tabstop=4 expandtab shiftwidth=4
