#pragma once

#include "pipe_accessor.h"
#include "pipe_object_interface.h"
#include "pipe_paging.h"
#include "pager.h"
#include "pages_conserving.h"

/* A pager abstraction for a pipe. */

class PipePager : public Pager, public PipeObject
{
protected:
    PipeAccessor _accessor;
    PagesConserving *_pages;
    PipePaging *_paging;
    bool _writer;

    /* Helper methods. */

    virtual long next_region_for_reader(offset_t *populated_size, offset_t *size);

    virtual long next_region_for_writer(offset_t *populated_size, offset_t *size);

public:
    explicit PipePager(PagesConserving *pages, PipePaging *paging, bool writer);

    /* Server details. */

    int expected_items();

    ipc_server_handler_type handler();

    void *interface()
    { return static_cast<PipeObject *>(this); }

    /* Pager methods. */

    virtual long map(unsigned long offset, l4_addr_t hot_spot, flags_t flags, l4_snd_fpage_t *region);

    /* Pipe methods. */

    virtual long current_region(offset_t *populated_size, offset_t *size);

    virtual long next_region(offset_t *populated_size, offset_t *size);
};

// vim: tabstop=4 expandtab shiftwidth=4
